==> Tekkotsu_2.0.tar.gz: Tekkotsu_2.0/tools/mon/org/tekkotsu/mon/VisionGUI.java <==
package org.tekkotsu.mon;

import java.awt.event.*;
import javax.swing.*;
import java.lang.String;
import java.util.LinkedList;
import java.awt.*;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.util.Date;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.util.prefs.Preferences;
import java.io.File;


public class VisionGUI extends JFrame implements ActionListener, VisionUpdatedListener {
	VisionPanel vision;
	JCheckBox aspectBut;
	JButton freezeBut;
	JButton saveImageBut;
	JLabel status;
	JButton reconnectBut;
	float mspf=0;
	float mspfGamma=.9f;
	long lastFrameTime=0;
	boolean isRaw=false;
	boolean isRLE=false;
	boolean connected=false;
	boolean isFreezeFrame=false;
	final static ImageIcon carrows = new ImageIcon("images/chasingarrows.png");
	ImageSequenceWriterThread imgWrite=null;
	String state="Connecting...";
	static Preferences prefs = Preferences.userNodeForPackage(VisionGUI.class);

	public class StatusUpdateThread extends Thread {
		VisionGUI gui;
		public StatusUpdateThread(VisionGUI gui) { super("StatusUpdate"); this.gui=gui; setDaemon(true);}
		public void run() {
			VisionListener l=gui.vision.getListener();
			while(true) {
				String status=gui.state;
				String recording=recordReport();
				String fps=fpsReport();
				if(fps.length()>0)
					status+=" - "+fps;
				if(recording.length()>0)
					status+="; "+recording;
				gui.status.setText(status);
				try {
					sleep(100);
				} catch(Exception ex) {break;}
			}
		}
		public String recordReport() {
			if(gui.imgWrite!=null && !gui.imgWrite.isAlive())
				gui.imgWrite=null;
			if(gui.imgWrite!=null && !gui.imgWrite.stopping) {
				String ans;
				if(!gui.imgWrite.stopping) {
					ans="Rec: ";
					if(imgWrite.imgBufs.size()!=0)
						ans+="Free="+imgWrite.imgBufs.size();
				} else
					ans="Writing";
				ans+=" Queue="+imgWrite.q.size();
				return ans;
			} else
				return "";
		}
		public String fpsReport() {
				if(connected) {
					int rnd=(int)(10000/gui.mspf);
					return rnd/10.0f+" fps";
				} else
					return "";
		}
	}
	
	public static void main(String s[]) {
		int port=-1;
		if(s.length<2)
			usage();
		String[] args=new String[1];
		if(s[1].toUpperCase().compareTo("RLE")==0) {
			args[0]=s[1];
			port=VisionListener.defRLEPort;
		} else if(s[1].toUpperCase().compareTo("RAW")==0) {
			args[0]=s[1];
			port=VisionListener.defRawPort;
		} else {
			args[0]="RAW";
			port=Integer.parseInt(s[1]);
		}
		if(s.length>2) {
			args[0]=s[2];
			if(s[2].toUpperCase().compareTo("RLE")==0)
				port=VisionListener.defRLEPort;
			else if(s[2].toUpperCase().compareTo("RAW")==0)
				port=VisionListener.defRawPort;
		}
		VisionGUI gui=new VisionGUI(s[0],port,args);
		gui.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) { System.exit(0); } });
	}
		
	public static void usage() {
		System.out.println("Usage: java VisionGUI host [port|raw|rle]");
		System.out.println("       if port is not specified, it defaults to:");
		System.out.println("       "+VisionListener.defRawPort+" for raw");
		System.out.println("       "+VisionListener.defRLEPort+" for RLE.");
		System.exit(2);
	}
			
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().compareTo("YUV")==0) {
			vision.setConvertRGB(false);
		} else if(e.getActionCommand().compareTo("RGB")==0) {
			vision.setConvertRGB(true);
		} else if(e.getActionCommand().compareTo("freeze")==0) {
			isFreezeFrame=!isFreezeFrame;
			if(!isFreezeFrame) {
				state="Reconnecting...";
				vision.open();
			} else {
				state="Disconnecting...";
				vision.close();
			}
		} else if(e.getActionCommand().compareTo("aspect")==0) {
			vision.setLockAspectRatio(((JCheckBox)e.getSource()).isSelected());
		} else if(e.getActionCommand().compareTo("seq")==0) {
			File cursavepath = new File(prefs.get("cursavepath",""));
			JFileChooser dia=new JFileChooser(cursavepath);
			dia.setDialogTitle("Save Image Sequence...");
			Component cur=this;
			while(cur.getParent()!=null)
				cur=cur.getParent();
			if(dia.showSaveDialog(cur)==JFileChooser.APPROVE_OPTION) {
				prefs.put("cursavepath",dia.getCurrentDirectory().getPath());
				imgWrite=new ImageSequenceWriterThread(vision.getListener(), saveImageBut, !vision.getConvertRGB());
				saveImageBut.setText("Stop Saving Sequence");
				saveImageBut.setToolTipText("Click to stop buffering new frames - already captured frames will continue to be written (unless you close the window)");
				saveImageBut.setActionCommand("stopseq");
				String base=dia.getSelectedFile().getName();
				String format;
				if(base.lastIndexOf('.')==-1) {
					format="png";
				} else {
					int i=base.lastIndexOf(".");
					format=base.substring(i+1);
					base=base.substring(0,i);
				}
				int first=base.indexOf('#');
				int last=base.lastIndexOf('#');
				boolean appendTime=(first==-1);
				imgWrite.setDirectory(dia.getSelectedFile().getParent());
				if(first!=-1)
					imgWrite.setName(base.substring(0,first),last-first+1,base.substring(last+1),format);
				else
					imgWrite.setName(base,0,"",format);
				imgWrite.start();
			}
		} else if(e.getActionCommand().compareTo("stopseq")==0) {
			saveImageBut.setText("Save Image Sequence");
			saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
			saveImageBut.setActionCommand("seq");
			if(imgWrite!=null)
				imgWrite.interrupt();
		} else if(e.getActionCommand().compareTo("img")==0) {
			File cursavepath = new File(prefs.get("cursavepath",""));
			JFileChooser dia=new JFileChooser(cursavepath);
			dia.setDialogTitle("Save Image...");
			Component cur=this;
			while(cur.getParent()!=null)
				cur=cur.getParent();
			if(dia.showSaveDialog(cur)==JFileChooser.APPROVE_OPTION) {
				prefs.put("cursavepath",dia.getCurrentDirectory().getPath());
				String base=dia.getSelectedFile().getName();
				String format;
				if(base.lastIndexOf('.')==-1) {
					format="png";
				} else {
					int i=base.lastIndexOf(".");
					format=base.substring(i+1);
					base=base.substring(0,i);
				}
				try {
					FileOutputStream fileout=new FileOutputStream(dia.getSelectedFile().getParent()+File.separator+base+"."+format);
					if(vision.getConvertRGB() && vision.getListener().hasRawJPEG() && (format.equalsIgnoreCase("jpg") || format.equalsIgnoreCase("jpeg"))) {
						byte[] jpeg=vision.getListener().getJPEG();
						int len=vision.getListener().getJPEGLen();
						fileout.write(jpeg,0,len);
					} else {
						ImageIO.write(vision.getListener().getImage(),format,fileout);
					}
				} catch(Exception ex) {ex.printStackTrace();}
			}
		} else if(e.getSource()==reconnectBut) {
			vision.close();
			vision.open();
		}
	}

	class CloseVisionAdapter extends WindowAdapter {
		VisionGUI gui;
		CloseVisionAdapter(VisionGUI gui) {this.gui=gui;}
		public void windowClosing(WindowEvent e) {
			gui.close();
		}
	}

	public void close() {
		String name="VisionGUI"+(isRaw?".raw":"")+(isRLE?".rle":"")+".location";
		prefs.putInt(name+".x",getLocation().x+getInsets().left);
		prefs.putInt(name+".y",getLocation().y+getInsets().top);
		this.vision.close();
		if(this.imgWrite!=null && this.imgWrite.isAlive()) {
			if(!this.imgWrite.isInterrupted() && !this.imgWrite.stopping)
				this.imgWrite.interrupt();
			while(this.imgWrite.isInterrupted())
				try { Thread.sleep(50); } catch(Exception ex) {}
			if(!this.imgWrite.stopping)
				System.out.println("imgWrite refuses to stop");
			this.imgWrite.interrupt(); //this second one should kill it
			while(this.imgWrite.isAlive())
				try { Thread.sleep(50); } catch(Exception ex) {}
		}
		vision.getListener().removeListener(this);
		vision.getListener().removeListener(vision);
		dispose();
	}
	public void visionUpdated(VisionListener l) {
		if(l.isConnected()!=connected) {
			connected=l.isConnected();
			if(connected) {
				freezeBut.setEnabled(true);
				freezeBut.setText("Freeze Frame");
				freezeBut.setToolTipText("Freezes current frame (disconnects from stream)");
				isFreezeFrame=false;
				saveImageBut.setEnabled(true);
				saveImageBut.setText("Save Image Sequence");
				saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
				saveImageBut.setActionCommand("seq");
				state="Connected.";
			} else {
				if(vision._image==null)
					saveImageBut.setEnabled(false);
				else {
					saveImageBut.setText("Save Image");
					saveImageBut.setToolTipText("Save current image shown - use .jpg or .png extension to choose format");
					saveImageBut.setActionCommand("img");
				}
				if(isFreezeFrame) {
					state="Disconnected.";
					freezeBut.setText("Unfreeze");
					freezeBut.setToolTipText("Reconnects to stream");
				} else {
					state="Reconnecting...";
					freezeBut.setEnabled(false);
				}
			}
		}
		if(connected) {
			if(lastFrameTime==0) {
				if(l.getTimeStamp()!=null)
					lastFrameTime=l.getTimeStamp().getTime();
			} else {
				long cur=l.getTimeStamp().getTime();
				mspf=mspf*mspfGamma+(cur-lastFrameTime)*(1-mspfGamma);
				lastFrameTime=cur;
			}
		} else {
			lastFrameTime=0;
		}
	}
	
	public VisionGUI(String host, String[] args) {
		super();
		int port=-1;
		for(int i=0; i<args.length; i++) {
			if(args[i].toUpperCase().compareTo("RLE")==0) {
				isRLE=true;
				isRaw=false;
			} else if(args[i].toUpperCase().compareTo("RAW")==0) {
				isRaw=true;
				isRLE=false;
			} else
				System.err.println("VisionGUI: Unrecognized argument: "+args[0]);
		}
		if(isRLE)
			port=VisionListener.defRLEPort;
		else if(isRaw)
			port=VisionListener.defRawPort;
		init(host,port,args);
	}
	public VisionGUI(String host, int port, String[] args) {
		super();
		init(host,port,args);
	}
	public void init(String host, int port, String[] args) {
		int strutsize=10;
		int sepsize=5;
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(Box.createVerticalStrut(strutsize),BorderLayout.NORTH);
		getContentPane().add(Box.createHorizontalStrut(strutsize),BorderLayout.WEST);
		getContentPane().add(Box.createHorizontalStrut(strutsize),BorderLayout.EAST);
		//always use RLE size for default - we'll assume it's full resolution
		vision=null;
		for(int i=0; i<args.length; i++) {
			if(args[i].toUpperCase().compareTo("RLE")==0) {
				setTitle("TekkotsuMon: Vision RLE");
				vision=new VisionPanel(new VisionListener(host,port));
				isRLE=true;
				isRaw=false;
			} else if(args[i].toUpperCase().compareTo("RAW")==0) {
				setTitle("TekkotsuMon: Vision Raw");
				vision=new VisionPanel(new VisionListener(host,port));
				isRaw=true;
				isRLE=false;
			} else
				System.err.println("VisionGUI: Unrecognized argument: "+args[i]);
		}
		if(vision==null) {
			System.err.println("VisionGUI: Error: Vision mode not specified");
		} else {
			vision.setMinimumSize(new Dimension(VisionListener.DEFAULT_WIDTH/2, VisionListener.DEFAULT_HEIGHT/2));
			vision.setPreferredSize(new Dimension(VisionListener.DEFAULT_WIDTH*2, VisionListener.DEFAULT_HEIGHT*2));
			vision.setLockAspectRatio(true);
			getContentPane().add(vision,BorderLayout.CENTER);
		}
		{
			Box tmp2=Box.createHorizontalBox();
			tmp2.add(Box.createHorizontalStrut(strutsize));
			{
				Box tmp3=Box.createVerticalBox();
				if(isRaw) {
					Box tmp4=Box.createHorizontalBox();
					ButtonGroup group=new ButtonGroup();
					JRadioButton tmpRad;
					tmpRad=new JRadioButton("RGB");
					tmpRad.setSelected(true);
					tmpRad.addActionListener(this);
					tmpRad.setToolTipText("Shows RGB colorspace");
					group.add(tmpRad);
					tmp4.add(tmpRad);
					tmpRad=new JRadioButton("YUV");
					tmpRad.addActionListener(this);
					tmpRad.setToolTipText("Shows YUV colorspace");
					group.add(tmpRad);
					tmp4.add(tmpRad);
					tmp3.add(tmp4);
				}
				aspectBut=new JCheckBox("Lock Aspect Ratio");
				aspectBut.setAlignmentX(0.5f);
				aspectBut.addActionListener(this);
				aspectBut.setActionCommand("aspect");
				aspectBut.setSelected(true);
				aspectBut.setToolTipText("Forces displayed image to hold transmitted image's aspect ratio");
				tmp3.add(aspectBut);
				freezeBut=new JButton("Freeze Frame");
				freezeBut.setAlignmentX(0.5f);
				freezeBut.addActionListener(this);
				freezeBut.setActionCommand("freeze");
				freezeBut.setEnabled(false);
				freezeBut.setToolTipText("Freezes current frame (disconnects from stream)");
				tmp3.add(freezeBut);
				saveImageBut=new JButton("Save Image Sequence");
				saveImageBut.setAlignmentX(0.5f);
				saveImageBut.addActionListener(this);
				saveImageBut.setActionCommand("seq");
				saveImageBut.setEnabled(false);
				saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
				tmp3.add(saveImageBut);
				tmp3.add(Box.createVerticalStrut(strutsize));
				tmp3.add(new JSeparator());
				tmp3.add(Box.createVerticalStrut(strutsize-sepsize));
				{
					Box tmp4=Box.createHorizontalBox();
					tmp4.add(status=new JLabel(state));
					tmp4.add(Box.createHorizontalGlue());
					reconnectBut=new JButton(carrows);
					reconnectBut.setPreferredSize(new Dimension(carrows.getIconWidth(),carrows.getIconHeight()));
					reconnectBut.addActionListener(this);
					reconnectBut.setToolTipText("Drop current connection and try again.");
					tmp4.add(reconnectBut);
					tmp3.add(tmp4);
				}
				tmp3.add(Box.createVerticalStrut(strutsize));
				tmp2.add(tmp3);
			}
			tmp2.add(Box.createHorizontalStrut(strutsize));
			getContentPane().add(tmp2,BorderLayout.SOUTH);
		}
		pack();
		addWindowListener(new CloseVisionAdapter(this));
		vision.getListener().addListener(this);
		(new StatusUpdateThread(this)).start();
		
		String name="VisionGUI"+(isRaw?".raw":"")+(isRLE?".rle":"")+".location";
		setLocation(prefs.getInt(name+".x",50),prefs.getInt(name+".y",50));

		show();

	}
}

==> Tekkotsu_2.1.tar.gz: Tekkotsu_2.1/tools/mon/org/tekkotsu/mon/VisionGUI.java <==
package org.tekkotsu.mon;

import java.awt.event.*;
import javax.swing.*;
import java.lang.String;
import java.util.LinkedList;
import java.awt.*;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.util.Date;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.util.prefs.Preferences;
import java.io.File;


public class VisionGUI extends JFrame implements ActionListener, VisionUpdatedListener {
	VisionPanel vision;
	JCheckBox aspectBut;
	JButton freezeBut;
	JButton saveImageBut;
	JLabel status;
	JButton reconnectBut;
	float mspf=0;
	float mspfGamma=.9f;
	long lastFrameTime=0;
	boolean isRaw=false;
	boolean isRLE=false;
	boolean connected=false;
	boolean isFreezeFrame=false;
	final static ImageIcon carrows = new ImageIcon("images/chasingarrows.png");
	ImageSequenceWriterThread imgWrite=null;
	String state="Connecting...";
	static Preferences prefs = Preferences.userNodeForPackage(VisionGUI.class);

	public class StatusUpdateThread extends Thread {
		VisionGUI gui;
		public StatusUpdateThread(VisionGUI gui) { super("StatusUpdate"); this.gui=gui; setDaemon(true);}
		public void run() {
			VisionListener l=gui.vision.getListener();
			while(true) {
				String status=gui.state;
				String recording=recordReport();
				String fps=fpsReport();
				if(fps.length()>0)
					status+=" - "+fps;
				if(recording.length()>0)
					status+="; "+recording;
				gui.status.setText(status);
				try {
					sleep(100);
				} catch(Exception ex) {break;}
			}
		}
		public String recordReport() {
			if(gui.imgWrite!=null && !gui.imgWrite.isAlive())
				gui.imgWrite=null;
			if(gui.imgWrite!=null && !gui.imgWrite.stopping) {
				String ans;
				if(!gui.imgWrite.stopping) {
					ans="Rec: ";
					if(imgWrite.imgBufs.size()!=0)
						ans+="Free="+imgWrite.imgBufs.size();
				} else
					ans="Writing";
				ans+=" Queue="+imgWrite.q.size();
				return ans;
			} else
				return "";
		}
		public String fpsReport() {
				if(connected) {
					int rnd=(int)(10000/gui.mspf);
					return rnd/10.0f+" fps";
				} else
					return "";
		}
	}
	
	public static void main(String s[]) {
		int port=-1;
		if(s.length<2)
			usage();
		String[] args=new String[1];
		if(s[1].toUpperCase().compareTo("RLE")==0) {
			args[0]=s[1];
			port=VisionListener.defRLEPort;
		} else if(s[1].toUpperCase().compareTo("RAW")==0) {
			args[0]=s[1];
			port=VisionListener.defRawPort;
		} else {
			port=Integer.parseInt(s[1]);
		}
		if(s.length>2) {
			args[0]=s[2];
			if(s[2].toUpperCase().compareTo("RLE")==0)
				port=VisionListener.defRLEPort;
			else if(s[2].toUpperCase().compareTo("RAW")==0)
				port=VisionListener.defRawPort;
		}
		VisionGUI gui;
		if(args[0].length()==0)
			gui=new VisionGUI(s[0],port,new String[0]);
		else
			gui=new VisionGUI(s[0],port,args);
		gui.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) { System.exit(0); } });
	}
		
	public static void usage() {
		System.out.println("Usage: java VisionGUI host [port|raw|rle]");
		System.out.println("       if port is not specified, it defaults to:");
		System.out.println("       "+VisionListener.defRawPort+" for raw");
		System.out.println("       "+VisionListener.defRLEPort+" for RLE.");
		System.exit(2);
	}
			
	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().compareTo("YUV")==0) {
			vision.setConvertRGB(false);
		} else if(e.getActionCommand().compareTo("RGB")==0) {
			vision.setConvertRGB(true);
		} else if(e.getActionCommand().compareTo("freeze")==0) {
			isFreezeFrame=!isFreezeFrame;
			if(!isFreezeFrame) {
				state="Reconnecting...";
				vision.open();
			} else {
				state="Disconnecting...";
				vision.close();
			}
		} else if(e.getActionCommand().compareTo("aspect")==0) {
			vision.setLockAspectRatio(((JCheckBox)e.getSource()).isSelected());
		} else if(e.getActionCommand().compareTo("seq")==0) {
			File cursavepath = new File(prefs.get("cursavepath",""));
			JFileChooser dia=new JFileChooser(cursavepath);
			dia.setDialogTitle("Save Image Sequence...");
			Component cur=this;
			while(cur.getParent()!=null)
				cur=cur.getParent();
			if(dia.showSaveDialog(cur)==JFileChooser.APPROVE_OPTION) {
				prefs.put("cursavepath",dia.getCurrentDirectory().getPath());
				imgWrite=new ImageSequenceWriterThread(vision.getListener(), saveImageBut, !vision.getConvertRGB());
				saveImageBut.setText("Stop Saving Sequence");
				saveImageBut.setToolTipText("Click to stop buffering new frames - already captured frames will continue to be written (unless you close the window)");
				saveImageBut.setActionCommand("stopseq");
				String base=dia.getSelectedFile().getName();
				String format;
				if(base.lastIndexOf('.')==-1) {
					format="png";
				} else {
					int i=base.lastIndexOf(".");
					format=base.substring(i+1);
					base=base.substring(0,i);
				}
				int first=base.indexOf('#');
				int last=base.lastIndexOf('#');
				boolean appendTime=(first==-1);
				imgWrite.setDirectory(dia.getSelectedFile().getParent());
				if(first!=-1)
					imgWrite.setName(base.substring(0,first),last-first+1,base.substring(last+1),format);
				else
					imgWrite.setName(base,0,"",format);
				imgWrite.start();
			}
		} else if(e.getActionCommand().compareTo("stopseq")==0) {
			saveImageBut.setText("Save Image Sequence");
			saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
			saveImageBut.setActionCommand("seq");
			if(imgWrite!=null)
				imgWrite.interrupt();
		} else if(e.getActionCommand().compareTo("img")==0) {
			File cursavepath = new File(prefs.get("cursavepath",""));
			JFileChooser dia=new JFileChooser(cursavepath);
			dia.setDialogTitle("Save Image...");
			Component cur=this;
			while(cur.getParent()!=null)
				cur=cur.getParent();
			if(dia.showSaveDialog(cur)==JFileChooser.APPROVE_OPTION) {
				prefs.put("cursavepath",dia.getCurrentDirectory().getPath());
				String base=dia.getSelectedFile().getName();
				String format;
				if(base.lastIndexOf('.')==-1) {
					format="png";
				} else {
					int i=base.lastIndexOf(".");
					format=base.substring(i+1);
					base=base.substring(0,i);
				}
				try {
					FileOutputStream fileout=new FileOutputStream(dia.getSelectedFile().getParent()+File.separator+base+"."+format);
					if(vision.getConvertRGB() && vision.getListener().hasRawJPEG() && (format.equalsIgnoreCase("jpg") || format.equalsIgnoreCase("jpeg"))) {
						byte[] jpeg=vision.getListener().getJPEG();
						int len=vision.getListener().getJPEGLen();
						fileout.write(jpeg,0,len);
					} else {
						ImageIO.write(vision.getListener().getImage(),format,fileout);
					}
				} catch(Exception ex) {ex.printStackTrace();}
			}
		} else if(e.getSource()==reconnectBut) {
			vision.close();
			vision.open();
		}
	}

	class CloseVisionAdapter extends WindowAdapter {
		VisionGUI gui;
		CloseVisionAdapter(VisionGUI gui) {this.gui=gui;}
		public void windowClosing(WindowEvent e) {
			gui.close();
		}
	}

	public void close() {
		String name="VisionGUI"+(isRaw?".raw":"")+(isRLE?".rle":"")+".location";
		prefs.putInt(name+".x",getLocation().x);
		prefs.putInt(name+".y",getLocation().y);
		//I think I had needed to add getInsets() to keep the window from moving when reopening it, but now it *causes* it to move... weird.  what changed?
		//prefs.putInt(name+".x",getLocation().x+getInsets().left);
		//prefs.putInt(name+".y",getLocation().y+getInsets().top);
		this.vision.close();
		if(this.imgWrite!=null && this.imgWrite.isAlive()) {
			if(!this.imgWrite.isInterrupted() && !this.imgWrite.stopping)
				this.imgWrite.interrupt();
			while(this.imgWrite.isInterrupted())
				try { Thread.sleep(50); } catch(Exception ex) {}
			if(!this.imgWrite.stopping)
				System.out.println("imgWrite refuses to stop");
			this.imgWrite.interrupt(); //this second one should kill it
			while(this.imgWrite.isAlive())
				try { Thread.sleep(50); } catch(Exception ex) {}
		}
		vision.getListener().removeListener(this);
		vision.getListener().removeListener(vision);
		dispose();
	}
	public void visionUpdated(VisionListener l) {
		if(l.isConnected()!=connected) {
			connected=l.isConnected();
			if(connected) {
				freezeBut.setEnabled(true);
				freezeBut.setText("Freeze Frame");
				freezeBut.setToolTipText("Freezes current frame (disconnects from stream)");
				isFreezeFrame=false;
				saveImageBut.setEnabled(true);
				saveImageBut.setText("Save Image Sequence");
				saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
				saveImageBut.setActionCommand("seq");
				state="Connected.";
			} else {
				if(vision._image==null)
					saveImageBut.setEnabled(false);
				else {
					saveImageBut.setText("Save Image");
					saveImageBut.setToolTipText("Save current image shown - use .jpg or .png extension to choose format");
					saveImageBut.setActionCommand("img");
				}
				if(isFreezeFrame) {
					state="Disconnected.";
					freezeBut.setText("Unfreeze");
					freezeBut.setToolTipText("Reconnects to stream");
				} else {
					state="Reconnecting...";
					freezeBut.setEnabled(false);
				}
			}
		}
		if(connected) {
			if(lastFrameTime==0) {
				if(l.getTimeStamp()!=null)
					lastFrameTime=l.getTimeStamp().getTime();
			} else {
				long cur=l.getTimeStamp().getTime();
				mspf=mspf*mspfGamma+(cur-lastFrameTime)*(1-mspfGamma);
				lastFrameTime=cur;
			}
		} else {
			lastFrameTime=0;
		}
	}
	
	public VisionGUI(String host, String[] args) {
		super();
		int port=-1;
		for(int i=0; i<args.length; i++) {
			if(args[i].toUpperCase().compareTo("RLE")==0) {
				isRLE=true;
				isRaw=false;
			} else if(args[i].toUpperCase().compareTo("RAW")==0) {
				isRaw=true;
				isRLE=false;
			} else
				System.err.println("VisionGUI: Unrecognized argument: "+args[0]);
		}
		if(isRLE)
			port=VisionListener.defRLEPort;
		else if(isRaw)
			port=VisionListener.defRawPort;
		init(host,port,args);
	}
	public VisionGUI(String host, int port, String[] args) {
		super();
		init(host,port,args);
	}
	public void init(String host, int port, String[] args) {
		int strutsize=10;
		int sepsize=5;
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(Box.createVerticalStrut(strutsize),BorderLayout.NORTH);
		getContentPane().add(Box.createHorizontalStrut(strutsize),BorderLayout.WEST);
		getContentPane().add(Box.createHorizontalStrut(strutsize),BorderLayout.EAST);
		//always use RLE size for default - we'll assume it's full resolution
		vision=null;
		for(int i=0; i<args.length; i++) {
			if(args[i].toUpperCase().compareTo("RLE")==0) {
				setTitle("TekkotsuMon: Vision RLE");
				vision=new VisionPanel(new VisionListener(host,port));
				isRLE=true;
				isRaw=false;
				break;
			} else if(args[i].toUpperCase().compareTo("RAW")==0) {
				setTitle("TekkotsuMon: Vision Raw");
				vision=new VisionPanel(new VisionListener(host,port));
				isRaw=true;
				isRLE=false;
				break;
			} else
				System.err.println("VisionGUI: Unrecognized argument: "+args[i]);
		}
		if(vision==null) {
			vision=new VisionPanel(new VisionListener(host,port));
			if(port==VisionListener.defRLEPort) {
				setTitle("TekkotsuMon: Vision RLE");
				isRLE=true;
				isRaw=false;
			} else {
				setTitle("TekkotsuMon: Vision Raw");
				isRLE=false;
				isRaw=true;
			}
		}
		vision.setMinimumSize(new Dimension(VisionListener.DEFAULT_WIDTH/2, VisionListener.DEFAULT_HEIGHT/2));
		vision.setPreferredSize(new Dimension(VisionListener.DEFAULT_WIDTH*2, VisionListener.DEFAULT_HEIGHT*2));
		vision.setLockAspectRatio(true);
		getContentPane().add(vision,BorderLayout.CENTER);
		{
			Box tmp2=Box.createHorizontalBox();
			tmp2.add(Box.createHorizontalStrut(strutsize));
			{
				Box tmp3=Box.createVerticalBox();
				if(isRaw) {
					Box tmp4=Box.createHorizontalBox();
					ButtonGroup group=new ButtonGroup();
					JRadioButton tmpRad;
					tmpRad=new JRadioButton("RGB");
					tmpRad.setSelected(true);
					tmpRad.addActionListener(this);
					tmpRad.setToolTipText("Shows RGB colorspace");
					group.add(tmpRad);
					tmp4.add(tmpRad);
					tmpRad=new JRadioButton("YUV");
					tmpRad.addActionListener(this);
					tmpRad.setToolTipText("Shows YUV colorspace");
					group.add(tmpRad);
					tmp4.add(tmpRad);
					tmp3.add(tmp4);
				}
				aspectBut=new JCheckBox("Lock Aspect Ratio");
				aspectBut.setAlignmentX(0.5f);
				aspectBut.addActionListener(this);
				aspectBut.setActionCommand("aspect");
				aspectBut.setSelected(true);
				aspectBut.setToolTipText("Forces displayed image to hold transmitted image's aspect ratio");
				tmp3.add(aspectBut);
				freezeBut=new JButton("Freeze Frame");
				freezeBut.setAlignmentX(0.5f);
				freezeBut.addActionListener(this);
				freezeBut.setActionCommand("freeze");
				freezeBut.setEnabled(false);
				freezeBut.setToolTipText("Freezes current frame (disconnects from stream)");
				tmp3.add(freezeBut);
				saveImageBut=new JButton("Save Image Sequence");
				saveImageBut.setAlignmentX(0.5f);
				saveImageBut.addActionListener(this);
				saveImageBut.setActionCommand("seq");
				saveImageBut.setEnabled(false);
				saveImageBut.setToolTipText("Saves to a series of files - use .jpg or .png extension to choose format; #'s will be replaced with index, otherwise timecode is appended");
				tmp3.add(saveImageBut);
				tmp3.add(Box.createVerticalStrut(strutsize));
				tmp3.add(new JSeparator());
				tmp3.add(Box.createVerticalStrut(strutsize-sepsize));
				{
					Box tmp4=Box.createHorizontalBox();
					tmp4.add(status=new JLabel(state));
					tmp4.add(Box.createHorizontalGlue());
					reconnectBut=new JButton(carrows);
					reconnectBut.setPreferredSize(new Dimension(carrows.getIconWidth(),carrows.getIconHeight()));
					reconnectBut.addActionListener(this);
					reconnectBut.setToolTipText("Drop current connection and try again.");
					tmp4.add(reconnectBut);
					tmp3.add(tmp4);
				}
				tmp3.add(Box.createVerticalStrut(strutsize));
				tmp2.add(tmp3);
			}
			tmp2.add(Box.createHorizontalStrut(strutsize));
			getContentPane().add(tmp2,BorderLayout.SOUTH);
		}
		pack();
		addWindowListener(new CloseVisionAdapter(this));
		vision.getListener().addListener(this);
		(new StatusUpdateThread(this)).start();
		
		String name="VisionGUI"+(isRaw?".raw":"")+(isRLE?".rle":"")+".location";
		setLocation(prefs.getInt(name+".x",50),prefs.getInt(name+".y",50));

		show();

	}
}
